#ifndef D3DDEVICEOBSERVERPATTERN_H
#define D3DDEVICEOBSERVERPATTERN_H

//--------------------------------------------------------------------------------------
// File: D3DDeviceObserverPattern.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// D3DDevice Observer pattern
// ID3DDeviceSubject and ID3DDeviceObserver interfaces
//
//--------------------------------------------------------------------------------------

class ID3DDeviceSubject;
class ID3DDeviceObserver;

//--------------------------------------------------------------------------------------
// ID3DDeviceSubject interface
//--------------------------------------------------------------------------------------
class ID3DDeviceSubject
{
public:
	ID3DDeviceSubject() { }
	virtual ~ID3DDeviceSubject() { }

	virtual void RegisterObserver(ID3DDeviceObserver * d3dDeviceObserver) = 0;
	virtual void RemoveObserver(ID3DDeviceObserver * d3dDeviceObserver) = 0;

	virtual HRESULT NotifyCreatedDevice( IDirect3DDevice9* pd3dDevice ) = 0;
	virtual HRESULT NotifyResetDevice() = 0;
	virtual HRESULT NotifyLostDevice() = 0;
	virtual HRESULT NotifyDestroyDevice() = 0;
};

//--------------------------------------------------------------------------------------
// ID3DDeviceObserver interface
//--------------------------------------------------------------------------------------
class ID3DDeviceObserver
{
public:
	ID3DDeviceObserver() { }
	virtual ~ID3DDeviceObserver() { }

	virtual HRESULT OnCreatedDevice( IDirect3DDevice9* pd3dDevice ) = 0;
	virtual HRESULT OnResetDevice() = 0;
	virtual HRESULT OnLostDevice() = 0;
	virtual HRESULT OnDestroyDevice() = 0;
};

#endif // D3DDEVICEOBSERVERPATTERN_H