//--------------------------------------------------------------------------------------
// File: D3DDeviceSubject.cpp
// Author: Peter Cerno <petercerno@gmail.com>
//
// D3DDevice Observer pattern
// Concrete class for ID3DDeviceSubject interface
//
//--------------------------------------------------------------------------------------

#include "dxstdafx.h"

#pragma warning(disable : 4530)

#include <algorithm>
#include <vector>

#include "D3DDeviceSubject.h"

//--------------------------------------------------------------------------------------
// CD3DDeviceSubject implements ID3DDeviceSubject interface
//--------------------------------------------------------------------------------------
void CD3DDeviceSubject::RegisterObserver(ID3DDeviceObserver *d3dDeviceObserver)
{
	if (d3dDeviceObserver &&
		std::find(
			m_D3DDeviceObserver.begin(),
			m_D3DDeviceObserver.end(),
			d3dDeviceObserver) == m_D3DDeviceObserver.end())
		m_D3DDeviceObserver.push_back(d3dDeviceObserver);
}

void CD3DDeviceSubject::RemoveObserver(ID3DDeviceObserver *d3dDeviceObserver)
{
	std::vector<ID3DDeviceObserver *>::iterator it =
		std::find(
			m_D3DDeviceObserver.begin(),
			m_D3DDeviceObserver.end(),
			d3dDeviceObserver);
	if (it != m_D3DDeviceObserver.end())
		m_D3DDeviceObserver.erase(it);
}

HRESULT CD3DDeviceSubject::NotifyCreatedDevice( IDirect3DDevice9* pd3dDevice )
{
	HRESULT hr = S_OK;
	size_t count = m_D3DDeviceObserver.size();
	size_t index = 0;
	for (index = 0; index < count; ++index)
		if (m_D3DDeviceObserver[index]->OnCreatedDevice(pd3dDevice) != S_OK)
			hr = E_FAIL;
	return hr;
}

HRESULT CD3DDeviceSubject::NotifyResetDevice()
{
	HRESULT hr = S_OK;
	size_t count = m_D3DDeviceObserver.size();
	size_t index = 0;
	for (index = 0; index < count; ++index)
		if (m_D3DDeviceObserver[index]->OnResetDevice() != S_OK)
			hr = E_FAIL;
	return hr;
}

HRESULT CD3DDeviceSubject::NotifyLostDevice()
{
	HRESULT hr = S_OK;
	size_t count = m_D3DDeviceObserver.size();
	size_t index = 0;
	for (index = 0; index < count; ++index)
		if (m_D3DDeviceObserver[index]->OnLostDevice() != S_OK)
			hr = E_FAIL;
	return hr;
}

HRESULT CD3DDeviceSubject::NotifyDestroyDevice()
{
	HRESULT hr = S_OK;
	size_t count = m_D3DDeviceObserver.size();
	size_t index = 0;
	for (index = 0; index < count; ++index)
		if (m_D3DDeviceObserver[index]->OnDestroyDevice() != S_OK)
			hr = E_FAIL;
	return hr;
}