//--------------------------------------------------------------------------------------
// File: Log.cpp
// Author: Peter Cerno <petercerno@gmail.com>
//
// Log for Enki::NeuralRobot source
// CLog class
//
//--------------------------------------------------------------------------------------

#pragma warning(disable : 4530)

#include <ios>
#include <iomanip>
#include <fstream>

#include "enki/robots/NeuralRobot.h"

#include "Log.h"

//--------------------------------------------------------------------------------------
// CLog class implements IReportErrorAble interface
//--------------------------------------------------------------------------------------
bool CLog::OpenLog(std::string fileName)
{
	m_LogFileCreated = false;
	m_Logging = false;
	if (m_log.is_open())
		m_log.close();
	std::ifstream inp;
	inp.open(fileName.c_str(), std::ifstream::in);
	inp.close();
	if (inp.fail())
	{
		inp.clear(std::ios::failbit);
		m_log.open(fileName.c_str());
		m_LogFileCreated = m_log.good();
		if (!m_LogFileCreated)
		{
			Error("Error in CLog: unable to create log file");
		}
	}
	else
	{
		Error("Error in CLog: log file already exists");
	}
	return m_LogFileCreated;
}

bool CLog::CloseLog()
{
	m_LogFileCreated = false;
	m_Logging = false;
	m_NeuralRobot = 0;
	if (m_log.is_open())
		m_log.close();
	return true;
}

bool CLog::Attach(Enki::NeuralRobot *NeuralRobot)
{
	m_Logging = false;
	m_NeuralRobot = NeuralRobot;
	return true;
}

bool CLog::CheckLogging()
{
	if (!m_NeuralRobot) { Error("Error in CLog: log is not attached to a robot"); return false; }
	if (!m_NeuralRobot->GetRemoteControl()) { Error("Error in CLog: remote control is not attached to a robot"); return false; }
	if (!m_LogFileCreated) { Error("Error in CLog: log file is not created yet"); return false; }
	return true;
}

bool CLog::StartLog()
{
	m_Logging = CheckLogging();
	return m_Logging;
}

bool CLog::PauseLog()
{
	m_Logging = false;
	return true;
}

void CLog::Log(double ElapsedTime)
{
	if (m_Logging)
	{
		if (!CheckLogging()) { m_Logging = false; return; }
		m_ElapsedTime += ElapsedTime;
		if (m_ElapsedTime > m_WaitingTime)
		{
			m_ElapsedTime = 0.0;
			size_t Count = 0;
			size_t Index = 0;
			Count = m_NeuralRobot->infraredSensors.size();
			for (Index = 0; Index < Count; ++Index)
			{
				m_log << std::setw(8) << std::setfill('0') <<
					m_NeuralRobot->infraredSensors[Index]->finalValue << ' ';
			}
			if (m_NeuralRobot->GetRemoteControl()->isUp())    m_log << "1.0 "; else m_log << "0.0 ";
			if (m_NeuralRobot->GetRemoteControl()->isDown())  m_log << "1.0 "; else m_log << "0.0 ";
			if (m_NeuralRobot->GetRemoteControl()->isLeft())  m_log << "1.0 "; else m_log << "0.0 ";
			if (m_NeuralRobot->GetRemoteControl()->isRight()) m_log << "1.0 "; else m_log << "0.0 ";
			m_log << std::endl;
		}
	}
}