#ifndef LOG_H
#define LOG_H

//--------------------------------------------------------------------------------------
// File: Log.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// Log for Enki::NeuralRobot header
// CLog class
//
//--------------------------------------------------------------------------------------

#include <fstream>

#include "RFWReportError.h"

namespace Enki
{
	class NeuralRobot;
}

class CLog;

//--------------------------------------------------------------------------------------
// CLog class implements IReportErrorAble interface
//--------------------------------------------------------------------------------------
class CLog: public RFW::IReportErrorAble
{
public:
	CLog():
	  m_LogFileCreated(false),
	  m_Logging(false),
	  m_NeuralRobot(0),
	  m_ElapsedTime(0.0),
	  m_WaitingTime(0.1) { }
	virtual ~CLog() { }

	virtual bool Good() { return m_ReportError.Good(); }
	virtual void Error(std::string err) { m_ReportError.Error(err); }
	virtual void ClearError() { m_ReportError.ClearError(); }
	virtual std::string ReportError() { return m_ReportError.ReportError(); }

	virtual bool OpenLog(std::string fileName);
	virtual bool CloseLog();

	virtual bool Attach(Enki::NeuralRobot * NeuralRobot);

	virtual bool StartLog();
	virtual bool PauseLog();
	virtual bool isLogging() { return m_Logging; }

	virtual void Log(double ElapsedTime);
protected:
	virtual bool CheckLogging();

	RFW::CReportError m_ReportError;

	bool m_LogFileCreated;
	bool m_Logging;
	std::ofstream m_log;
	Enki::NeuralRobot * m_NeuralRobot;

	double m_ElapsedTime;
	double m_WaitingTime;
};

#endif // LOG_H