//--------------------------------------------------------------------------------------
// File: MsgSubject.cpp
// Author: Peter Cerno <petercerno@gmail.com>
//
// MsgProc Observer pattern
// Concrete class for IMsgSubject interface
//
//--------------------------------------------------------------------------------------

#include "dxstdafx.h"

#pragma warning(disable : 4530)

#include <algorithm>
#include <vector>

#include "MsgSubject.h"

//--------------------------------------------------------------------------------------
// CMsgSubject implements IMsgSubject interface
//--------------------------------------------------------------------------------------
void CMsgSubject::RegisterObserver(IMsgObserver *msgObserver)
{
	if (msgObserver &&
		std::find(m_MsgObserver.begin(), m_MsgObserver.end(), msgObserver) == m_MsgObserver.end())
		m_MsgObserver.push_back(msgObserver);
}

void CMsgSubject::RemoveObserver(IMsgObserver *msgObserver)
{
	std::vector<IMsgObserver *>::iterator it =
		std::find(m_MsgObserver.begin(), m_MsgObserver.end(), msgObserver);
	if (it != m_MsgObserver.end())
		m_MsgObserver.erase(it);
}

void CMsgSubject::NotifyObservers(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	size_t count = m_MsgObserver.size();
	size_t index = 0;
	for (index = 0; index < count; ++index)
		m_MsgObserver[index]->MsgProc(hWnd, uMsg, wParam, lParam);
}