#ifndef RFWCORE_H
#define RFWCORE_H

//--------------------------------------------------------------------------------------
// File: RFWCore.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// Robot Framework core header
// Contains all basic interfaces
//
//--------------------------------------------------------------------------------------

namespace RFW
{
	class IWorld;
	class IPhysicalObject;
	class IRobot;
}

namespace RFW
{
	//--------------------------------------------------------------------------------------
	// IWorld interface
	//--------------------------------------------------------------------------------------
	class IWorld
	{
	protected:
		IWorld(const IWorld &);
		const IWorld & operator=(const IWorld &);
	public:
		IWorld() { }
		virtual ~IWorld() { }

		virtual void ClearWorld(double, double) = 0;
		
		virtual size_t GetPhysicalObjectCount() = 0;
		virtual IPhysicalObject * GetPhysicalObject(size_t) = 0;

		virtual size_t GetRobotCount() = 0;
		virtual IRobot * GetRobot(size_t) = 0;

		virtual void Step(double) = 0;
	};

	//--------------------------------------------------------------------------------------
	// IPhysicalObject interface
	//--------------------------------------------------------------------------------------
	class IPhysicalObject
	{
	protected:
		IPhysicalObject(const IPhysicalObject &);
		const IPhysicalObject & operator=(const IPhysicalObject &);
	public:
		IPhysicalObject() { }
		virtual ~IPhysicalObject() { }
	};

	//--------------------------------------------------------------------------------------
	// IRobot interface
	//--------------------------------------------------------------------------------------
	class IRobot
	{
	protected:
		IRobot(const IRobot &);
		const IRobot & operator=(const IRobot &);
	public:
		IRobot() { }
		virtual ~IRobot() { }
	};
}

#endif // RFWCORE_H