#ifndef RFWCOREGRAPHICSCLASSES_H
#define RFWCOREGRAPHICSCLASSES_H

//--------------------------------------------------------------------------------------
// File: RFWCoreGraphicsClasses.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// Robot Framework core graphics header
// Graphics classes for CEnkiWorld, CEnkiPhysicalObject, CEnkiRobot classes
//
//--------------------------------------------------------------------------------------

#include "RFWCoreGraphics.h"

namespace Enki
{
	class PhysicalObject;
	class CircularCam;
	class IRSensor;
}

class CVertexBuffer;

namespace RFW
{
	class CEnkiWorld;
	class CEnkiPhysicalObject;
	class CEnkiRobot;

	class CEnkiPhysicalObjectPlotter;
	class CEnkiCircularCamPlotter;
	class CEnkiIRSensorPlotter;

	class CEnkiWorldGraphics;
	class CEnkiPhysicalObjectGraphics;
	class CEnkiRobotGraphics;
}

namespace RFW
{
	//--------------------------------------------------------------------------------------
	// CEnkiPhysicalObjectPlotter class plots Enki::PhysicalObject
	//--------------------------------------------------------------------------------------
	class CEnkiPhysicalObjectPlotter
	{
	protected:
		CEnkiPhysicalObjectPlotter(const CEnkiPhysicalObjectPlotter &);
		const CEnkiPhysicalObjectPlotter & operator=(const CEnkiPhysicalObjectPlotter &);
	public:
		explicit CEnkiPhysicalObjectPlotter(CVertexBuffer & vertexBuffer):
		  m_VertexBuffer(vertexBuffer) { }
		virtual ~CEnkiPhysicalObjectPlotter() { }

		virtual HRESULT Render(Enki::PhysicalObject & physicalObject);
	protected:
		CVertexBuffer & m_VertexBuffer;
	};

	//--------------------------------------------------------------------------------------
	// CEnkiCircularCamPlotter class plots Enki::CircularCam
	//--------------------------------------------------------------------------------------
	class CEnkiCircularCamPlotter
	{
	protected:
		CEnkiCircularCamPlotter(const CEnkiCircularCamPlotter &);
		const CEnkiCircularCamPlotter & operator=(const CEnkiCircularCamPlotter &);
	public:
		explicit CEnkiCircularCamPlotter(CVertexBuffer & vertexBuffer):
		  m_VertexBuffer(vertexBuffer) { }
		virtual ~CEnkiCircularCamPlotter() { }

		virtual HRESULT Render(Enki::CircularCam & circularCam, double r);
	protected:
		CVertexBuffer & m_VertexBuffer;
	};

	//--------------------------------------------------------------------------------------
	// CEnkiIRSensorPlotter class plots Enki::IRSensor
	//--------------------------------------------------------------------------------------
	class CEnkiIRSensorPlotter
	{
	protected:
		CEnkiIRSensorPlotter(const CEnkiIRSensorPlotter &);
		const CEnkiIRSensorPlotter & operator=(const CEnkiIRSensorPlotter &);
	public:
		explicit CEnkiIRSensorPlotter(CVertexBuffer & vertexBuffer):
		  m_VertexBuffer(vertexBuffer) { }
		virtual ~CEnkiIRSensorPlotter() { }

		virtual HRESULT Render(Enki::IRSensor & irSensor);
	protected:
		CVertexBuffer & m_VertexBuffer;
	};

	//--------------------------------------------------------------------------------------
	// CEnkiWorldGraphics class implements IEnkiGraphics interface for CEnkiWorld
	//--------------------------------------------------------------------------------------
	class CEnkiWorldGraphics: public IEnkiGraphics
	{
	protected:
		CEnkiWorldGraphics(const CEnkiWorldGraphics &);
		const CEnkiWorldGraphics & operator=(const CEnkiWorldGraphics &);
	public:
		explicit CEnkiWorldGraphics(CEnkiWorld & enkiWorld);
		virtual ~CEnkiWorldGraphics() { }

		virtual void Render();
	protected:
		CVertexBuffer & m_VertexBuffer;
		CEnkiWorld & m_EnkiWorld;
	};

	//---------------------------------------------------------------------------------------------
	// CEnkiPhysicalObjectGraphics class implements IEnkiGraphics interface for CEnkiPhysicalObject
	//---------------------------------------------------------------------------------------------
	class CEnkiPhysicalObjectGraphics: public IEnkiGraphics
	{
	protected:
		CEnkiPhysicalObjectGraphics(const CEnkiPhysicalObjectGraphics &);
		const CEnkiPhysicalObjectGraphics & operator=(const CEnkiPhysicalObjectGraphics &);
	public:
		explicit CEnkiPhysicalObjectGraphics(CEnkiPhysicalObject & enkiPhysicalObject);
		virtual ~CEnkiPhysicalObjectGraphics() { }

		virtual void Render();
	protected:
		CVertexBuffer & m_VertexBuffer;
		CEnkiPhysicalObject & m_EnkiPhysicalObject;

		CEnkiPhysicalObjectPlotter m_EnkiPhysicalObjectPlotter;
	};

	//--------------------------------------------------------------------------------------
	// CEnkiRobotGraphics class implements IEnkiGraphics interface for CEnkiRobot
	//--------------------------------------------------------------------------------------
	class CEnkiRobotGraphics: public IEnkiGraphics
	{
	protected:
		CEnkiRobotGraphics(const CEnkiRobotGraphics &);
		const CEnkiRobotGraphics & operator=(const CEnkiRobotGraphics &);
	public:
		explicit CEnkiRobotGraphics(CEnkiRobot & enkiRobot);
		virtual ~CEnkiRobotGraphics() { }

		virtual void Render();
	protected:
		CVertexBuffer & m_VertexBuffer;
		CEnkiRobot & m_EnkiRobot;

		CEnkiPhysicalObjectPlotter m_EnkiPhysicalObjectPlotter;
	};
}

#endif // RFWCOREGRAPHICSCLASSES_H