#ifndef RFWCOREIO_H
#define RFWCOREIO_H

//--------------------------------------------------------------------------------------
// File: RFWCoreIO.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// Robot Framework core IO header
// IO interfaces based on Enki
//
//--------------------------------------------------------------------------------------

#include "RFWReportError.h"

#import <msxml2.dll> 

namespace RFW
{
	class CEnkiIOStruct;
	class IEnkiIO;
}

namespace RFW
{
	//--------------------------------------------------------------------------------------
	// CEnkiIOStruct class
	//--------------------------------------------------------------------------------------
	class CEnkiIOStruct: public IReportErrorAble
	{
	public:
		CEnkiIOStruct() { }
		CEnkiIOStruct(MSXML2::IXMLDOMNodePtr nodePtr): m_nodePtr(nodePtr) { }
		CEnkiIOStruct(std::string fileName);
		virtual ~CEnkiIOStruct() { }

		virtual bool Good() { return m_ReportError.Good(); }
		virtual void Error(std::string err) { m_ReportError.Error(err); }
		virtual void ClearError() { m_ReportError.ClearError(); }
		virtual std::string ReportError() { return m_ReportError.ReportError(); }

		CEnkiIOStruct & SetNodePtr(MSXML2::IXMLDOMNodePtr nodePtr) { m_nodePtr = nodePtr; return *this; }
		MSXML2::IXMLDOMNodePtr & GetNodePtr() { return m_nodePtr; }
	protected:
		MSXML2::IXMLDOMNodePtr m_nodePtr;
		CReportError m_ReportError;
	};

	inline CEnkiIOStruct::CEnkiIOStruct(std::string fileName)
	{
		HRESULT                    hr;
		MSXML2::IXMLDOMDocumentPtr docPtr;
		MSXML2::IXMLDOMNodeListPtr listPtr;

		hr = docPtr.CreateInstance("Msxml.DOMDocument");
		// If "cannot instantiate Msxml.DOMDocument" error occurs then
		// it is possible that you have forgotten to call
		// CoInitialize(NULL); at the start of your application
		if (hr != S_OK) { Error("Error in CEnkiIOStruct: cannot instantiate Msxml.DOMDocument"); return; }
		// This error occurs if the file does not exist or it has bad xml format
		if (!docPtr->load(_bstr_t(fileName.c_str()))) { Error("Error in CEnkiIOStruct: cannot open the file"); return; }

		listPtr = docPtr->childNodes;
		if (listPtr->length == 1)
			m_nodePtr = listPtr->Getitem(0);
		else { Error("Error in CEnkiIOStruct: there is not one root"); return; }
	}

	//--------------------------------------------------------------------------------------
	// IEnkiIO interface
	//--------------------------------------------------------------------------------------
	class IEnkiIO
	{
	protected:
		IEnkiIO(const IEnkiIO &);
		const IEnkiIO & operator=(const IEnkiIO &);
	public:
		IEnkiIO() { }
		virtual ~IEnkiIO() { }

		virtual void Read(CEnkiIOStruct &) = 0;
	};
}

#endif // RFWCOREIO_H
