#ifndef RFWCOREIOCLASSES_H
#define RFWCOREIOCLASSES_H

//--------------------------------------------------------------------------------------
// File: RFWCoreIOClasses.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// Robot Framework core IO header
// IO classes for IWorld, IPhysicalObject, IRobot based on Enki
//
//--------------------------------------------------------------------------------------

#include "RFWCoreIO.h"

namespace Enki
{
	class PhysicalObject;
	class CircularCam;
	class IRSensor;
}

namespace RFW
{
	class CEnkiWorld;
	class CEnkiPhysicalObject;
	class CEnkiRobot;

	class CEnkiPhysicalObjectReader;
	class CEnkiCircularCamReader;
	class CEnkiIRSensorReader;

	class CEnkiWorldIO;
	class CEnkiPhysicalObjectIO;
	class CEnkiRobotIO;
}

namespace RFW
{
	//--------------------------------------------------------------------------------------
	// CAnPolygoneReader class reads An::Polygone
	//--------------------------------------------------------------------------------------
	class CAnPolygoneReader
	{
	protected:
		CAnPolygoneReader(const CAnPolygoneReader &);
		const CAnPolygoneReader & operator=(const CAnPolygoneReader &);
	public:
		CAnPolygoneReader() { }
		virtual ~CAnPolygoneReader() { }

		virtual void Read(CEnkiIOStruct & enkiIOStruct, An::Polygone & polygone);
	};

	//--------------------------------------------------------------------------------------
	// CEnkiPhysicalObjectReader class reads Enki::PhysicalObject
	//--------------------------------------------------------------------------------------
	class CEnkiPhysicalObjectReader
	{
	protected:
		CEnkiPhysicalObjectReader(const CEnkiPhysicalObjectReader &);
		const CEnkiPhysicalObjectReader & operator=(const CEnkiPhysicalObjectReader &);
	public:
		CEnkiPhysicalObjectReader() { }
		virtual ~CEnkiPhysicalObjectReader() { }

		virtual void Read(CEnkiIOStruct & enkiIOStruct, Enki::PhysicalObject & physicalObject);
	protected:
		CAnPolygoneReader m_AnPolygoneReader;
	};

	//--------------------------------------------------------------------------------------
	// CEnkiCircularCamReader class reads Enki::CircularCam
	//--------------------------------------------------------------------------------------
	class CEnkiCircularCamReader
	{
	protected:
		CEnkiCircularCamReader(const CEnkiCircularCamReader &);
		const CEnkiCircularCamReader & operator=(const CEnkiCircularCamReader &);
	public:
		CEnkiCircularCamReader() { }
		virtual ~CEnkiCircularCamReader() { }

		virtual void Read(CEnkiIOStruct & enkiIOStruct, Enki::CircularCam & circularCam);
	};

	//--------------------------------------------------------------------------------------
	// CEnkiIRSensorReader class reads Enki::IRSensor
	//--------------------------------------------------------------------------------------
	class CEnkiIRSensorReader
	{
	protected:
		CEnkiIRSensorReader(const CEnkiIRSensorReader &);
		const CEnkiIRSensorReader & operator=(const CEnkiIRSensorReader &);
	public:
		CEnkiIRSensorReader() { }
		virtual ~CEnkiIRSensorReader() { }

		virtual void Read(CEnkiIOStruct & enkiIOStruct, Enki::IRSensor & irSensor);
	};

	//--------------------------------------------------------------------------------------
	// CEnkiWorldIO class implements IEnkiIO interface for CEnkiWorld
	//--------------------------------------------------------------------------------------
	class CEnkiWorldIO: public IEnkiIO
	{
	protected:
		CEnkiWorldIO(const CEnkiWorldIO &);
		const CEnkiWorldIO & operator=(const CEnkiWorldIO &);
	public:
		explicit CEnkiWorldIO(CEnkiWorld & enkiWorld): m_EnkiWorld(enkiWorld) { }
		virtual ~CEnkiWorldIO() { }

		virtual void Read(CEnkiIOStruct &);
	protected:
		CEnkiWorld & m_EnkiWorld;
	};

	//--------------------------------------------------------------------------------------
	// CEnkiPhysicalObjectIO class implements IEnkiIO interface for CEnkiPhysicalObject
	//--------------------------------------------------------------------------------------
	class CEnkiPhysicalObjectIO: public IEnkiIO
	{
	protected:
		CEnkiPhysicalObjectIO(const CEnkiPhysicalObjectIO &);
		const CEnkiPhysicalObjectIO & operator=(const CEnkiPhysicalObjectIO &);
	public:
		explicit CEnkiPhysicalObjectIO(CEnkiPhysicalObject & enkiPhysicalObject):
		  m_EnkiPhysicalObject(enkiPhysicalObject) { }
		virtual ~CEnkiPhysicalObjectIO() { }

		virtual void Read(CEnkiIOStruct &);
	protected:
		CEnkiPhysicalObject & m_EnkiPhysicalObject;
		CEnkiPhysicalObjectReader m_EnkiPhysicalObjectReader;
	};

	//--------------------------------------------------------------------------------------
	// CEnkiRobotIO class implements IEnkiIO interface for CEnkiRobot
	//--------------------------------------------------------------------------------------
	class CEnkiRobotIO: public IEnkiIO
	{
	protected:
		CEnkiRobotIO(const CEnkiRobotIO &);
		const CEnkiRobotIO & operator=(const CEnkiRobotIO &);
	public:
		explicit CEnkiRobotIO(CEnkiRobot & enkiRobot):
		  m_EnkiRobot(enkiRobot) { }
		virtual ~CEnkiRobotIO() { }

		virtual void Read(CEnkiIOStruct &);
	protected:
		CEnkiRobot & m_EnkiRobot;
		CEnkiPhysicalObjectReader m_EnkiPhysicalObjectReader;
	};
}

#endif // RFWCOREIOCLASSES_H