#ifndef RFWNEURALNETWORK_H
#define RFWNEURALNETWORK_H

//--------------------------------------------------------------------------------------
// File: RFWNeuralNetwork.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// Robot Framework core classes extension header
// CFeedForward class for simulating feed-forward neural networks
//
//--------------------------------------------------------------------------------------

#include <vector>

#include "RFWAlgebra.h"

namespace RFW
{
	class CFeedForward;
	class CFeedForwardReader;
}

namespace RFW
{
	//--------------------------------------------------------------------------------------
	// CFeedForward class for simulating feed-forward neural networks
	// WARNING: This class should not be subclassed
	//--------------------------------------------------------------------------------------
	class CFeedForward
	{
	protected:
		CFeedForward(const CFeedForward &);
		const CFeedForward & operator=(const CFeedForward &);
	public:
		typedef double function(double);

		CFeedForward():
		  m_InputDimensions(0),
		  m_OutputDimensions(0),
		  m_NumLayers(0),
		  m_readMean(false),
		  m_readStd(false),
		  m_readPCAMatrix(false) { }
		virtual ~CFeedForward() { }

		void Clear();

		int InputDimensions() { return m_InputDimensions; }
		int OutputDimensions() { return m_OutputDimensions; }
		int NumLayers() { return m_NumLayers; }

		CVector Simulate(const CVector & x) const;
	protected:
		static double logsig(double x);
		static double tansig(double x);
		static double purelin(double x);

		bool CheckConsistency() const;

		int m_InputDimensions;
		int m_OutputDimensions;
		int m_NumLayers;
		CVector m_Mean;
		CVector m_Std;
		CMatrix m_PCAMatrix;
		std::vector<int> m_LayersDimensions;
		std::vector<function *> m_LayersTransferFcn;
		std::vector<CMatrix> m_Weights;
		std::vector<CVector> m_Biases;

		bool m_readMean;
		bool m_readStd;
		bool m_readPCAMatrix;

		friend CFeedForwardReader;
	};
}

#endif // RFWNEURALNETWORK_H