#ifndef RFWNEURALNETWORKIO_H
#define RFWNEURALNETWORKIO_H

//--------------------------------------------------------------------------------------
// File: RFWNeuralNetworkIO.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// Robot Framework core IO extension header
// CFeedForwardIO class for CFeedForward
//
//--------------------------------------------------------------------------------------

#include "RFWCoreIO.h"
#include "RFWAlgebraIO.h"

namespace RFW
{
	class CFeedForward;
	class CFeedForwardReader;
}

namespace RFW
{
	//--------------------------------------------------------------------------------------
	// CFeedForwardReader class reads RFW::CFeedForward
	//--------------------------------------------------------------------------------------
	class CFeedForwardReader
	{
	protected:
		CFeedForwardReader(const CFeedForwardReader &);
		const CFeedForwardReader & operator=(const CFeedForwardReader &);
	public:
		CFeedForwardReader() { }
		virtual ~CFeedForwardReader() { }

		virtual void Read(CEnkiIOStruct & enkiIOStruct, CFeedForward & feedForward);
	protected:
		CVectorReader m_VectorReader;
		CMatrixReader m_MatrixReader;
	protected:
		bool ReadMean(CEnkiIOStruct & enkiIOStruct, CFeedForward & feedForward);
		bool ReadStd(CEnkiIOStruct & enkiIOStruct, CFeedForward & feedForward);
		bool ReadPCAMatrix(CEnkiIOStruct & enkiIOStruct, CFeedForward & feedForward);
		bool ReadLayers(CEnkiIOStruct & enkiIOStruct, CFeedForward & feedForward);
		int  ReadLayer(CEnkiIOStruct & enkiIOStruct, CFeedForward & feedForward);
		bool ReadWeights(CEnkiIOStruct & enkiIOStruct, CFeedForward & feedForward);
		int  ReadWeight(CEnkiIOStruct & enkiIOStruct, CFeedForward & feedForward);
		bool ReadBiases(CEnkiIOStruct & enkiIOStruct, CFeedForward & feedForward);
		int  ReadBias(CEnkiIOStruct & enkiIOStruct, CFeedForward & feedForward);

		bool CheckTransferFcn(std::string TransferFcn);
		CFeedForward::function * GetTransferFcn(std::string TransferFcn);
	};
}


#endif // RFWNEURALNETWORKIO_H