//--------------------------------------------------------------------------------------
// File: RFWNeuralRobot.cpp
// Author: Peter Cerno <petercerno@gmail.com>
//
// Robot Framework core classes extension source
// Implementation of CEnkiNeuralRobot class
//
//--------------------------------------------------------------------------------------

#include "dxstdafx.h"

#pragma warning(disable : 4530)

#include "enki/PhysicalEngine.h"
#include "enki/robots/NeuralRobot.h"

#include "RFWNeuralRobot.h"
#include "RFWNeuralRobotGraphics.h"
#include "RFWNeuralRobotIO.h"

namespace RFW
{
	//--------------------------------------------------------------------------------------
	// CEnkiNeuralRobot class implements IEnkiRobot interface
	//--------------------------------------------------------------------------------------
	CEnkiNeuralRobot::CEnkiNeuralRobot(RFW::CEnkiWorld &enkiWorld):
		m_VertexBuffer(enkiWorld.m_VertexBuffer),
		m_EnkiWorld(enkiWorld)
	{
		ClearRobot();
		m_EnkiNeuralRobotGraphics = new CEnkiNeuralRobotGraphics(*this);
		m_EnkiNeuralRobotIO = new CEnkiNeuralRobotIO(*this);
		enkiWorld.AddRobot(*this); 
	}

	CEnkiNeuralRobot::~CEnkiNeuralRobot()
	{
		ClearRobot();
		delete m_EnkiNeuralRobotGraphics;
		delete m_EnkiNeuralRobotIO;
	}

	void CEnkiNeuralRobot::ClearRobot()
	{
		m_NeuralRobot.Clear();
	}

	void CEnkiNeuralRobot::Render()
	{
		m_EnkiNeuralRobotGraphics->Render();
	}

	void CEnkiNeuralRobot::Read(RFW::CEnkiIOStruct &enkiIOStruct)
	{
		m_EnkiNeuralRobotIO->Read(enkiIOStruct);
	}
}