#ifndef RFWREPORTERROR_H
#define RFWREPORTERROR_H

//--------------------------------------------------------------------------------------
// File: RFWReportError.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// Robot Framework report error header
// Classes for IReportErrorAble interface
//
//--------------------------------------------------------------------------------------

#pragma warning(disable : 4530)

#include <string>

namespace RFW
{
	class IReportErrorAble;
	class CReportError;
}

namespace RFW
{
	//--------------------------------------------------------------------------------------
	// IReportErrorAble interface
	//--------------------------------------------------------------------------------------
	class IReportErrorAble
	{
	public:
		IReportErrorAble() { }
		virtual ~IReportErrorAble() { }

		virtual bool Good() = 0;
		virtual void Error(std::string) = 0;
		virtual void ClearError() = 0;
		virtual std::string ReportError() = 0;
	};

	//--------------------------------------------------------------------------------------
	// CReportError class
	//--------------------------------------------------------------------------------------
	class CReportError: public IReportErrorAble
	{
	protected:
		CReportError(const CReportError &);
		const CReportError & operator=(const CReportError &);
	public:
		CReportError(): m_Good(true) { }
		virtual ~CReportError() { }

		virtual bool Good() { return m_Good; }
		virtual void Error(std::string err) { m_Error += err; m_Error.push_back('\n'); m_Good = false; }
		virtual void ClearError() { m_Error = ""; m_Good = true; }
		virtual std::string ReportError() { return m_Error; }
	protected:
		bool m_Good;
		std::string m_Error;
	};
}

#endif // RFWREPORTERROR_H