//--------------------------------------------------------------------------------------
// File: RemoteControl.cpp
// Author: Peter Cerno <petercerno@gmail.com>
//
// Remote control classes source
// Classes that extends Enki::RemoteControl class
//
//--------------------------------------------------------------------------------------

#include "dxstdafx.h"

#pragma warning(disable : 4530)

#include "RemoteControl.h"
#include "Keyboard.h"

//--------------------------------------------------------------------------------------
// CKeyboardRemoteControl class extends Enki::RemoteControl class
//--------------------------------------------------------------------------------------
bool CKeyboardRemoteControl::isUp() { return m_Keyboard.isKey(VK_UP); }
bool CKeyboardRemoteControl::isDown() { return m_Keyboard.isKey(VK_DOWN); }
bool CKeyboardRemoteControl::isLeft() { return m_Keyboard.isKey(VK_LEFT); }
bool CKeyboardRemoteControl::isRight() { return m_Keyboard.isKey(VK_RIGHT); }

//--------------------------------------------------------------------------------------
// CNeuralNetworkRemoteControl extends Enki::RemoteControl class
//--------------------------------------------------------------------------------------
bool CNeuralNetworkRemoteControl::isUp() { return bUp; }
bool CNeuralNetworkRemoteControl::isDown() { return bDown; }
bool CNeuralNetworkRemoteControl::isLeft() { return bLeft; }
bool CNeuralNetworkRemoteControl::isRight() { return bRight; }

void CNeuralNetworkRemoteControl::Clear()
{
	m_FeedForward.Clear();
}

void CNeuralNetworkRemoteControl::Update()
{
	if (IsConsistent())
	{
		int Count = (int)owner->infraredSensors.size();
		int Index = 0;
		RFW::CVector input(Count);
		RFW::CVector output;
		for (Index = 0; Index < Count; ++Index)
			input.SetAt(Index, owner->infraredSensors[Index]->finalValue);
		output = m_FeedForward.Simulate(input);
		if (output.GetM() == 4)
		{
			if (output.GetAt(0) > 0.5) bUp    = true; else bUp    = false;
			if (output.GetAt(1) > 0.5) bDown  = true; else bDown  = false;
			if (output.GetAt(2) > 0.5) bLeft  = true; else bLeft  = false;
			if (output.GetAt(3) > 0.5) bRight = true; else bRight = false;
		}
	}
	else
	{
		bUp = false;
		bDown = false;
		bLeft = false;
		bRight = false;
	}
}

bool CNeuralNetworkRemoteControl::IsConsistent()
{
	if (owner) return m_FeedForward.InputDimensions() == (int)owner->infraredSensors.size();
	else return false;
}