#ifndef REMOTECONTROL_H
#define REMOTECONTROL_H

//--------------------------------------------------------------------------------------
// File: RemoteControl.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// Remote control classes header
// Classes that extends Enki::RemoteControl class
//
//--------------------------------------------------------------------------------------

#include "enki/robots/NeuralRobot.h"
#include "RFWNeuralNetwork.h"

class CKeyboard;

class CKeyboardRemoteControl;
class CNeuralNetworkRemoteControl;

//--------------------------------------------------------------------------------------
// CKeyboardRemoteControl class extends Enki::RemoteControl class
//--------------------------------------------------------------------------------------
class CKeyboardRemoteControl: public Enki::RemoteControl
{
protected:
	CKeyboardRemoteControl(const CKeyboardRemoteControl &);
	const CKeyboardRemoteControl & operator=(const CKeyboardRemoteControl &);
public:
	CKeyboardRemoteControl(CKeyboard & keyboard):
	  m_Keyboard(keyboard) { }
	virtual ~CKeyboardRemoteControl() { }

	virtual bool isUp();
	virtual bool isDown();
	virtual bool isLeft();
	virtual bool isRight();

	virtual void Clear()  { }
	virtual void Update() { }
protected:
	CKeyboard & m_Keyboard;
};

//--------------------------------------------------------------------------------------
// CNeuralNetworkRemoteControl extends Enki::RemoteControl class
//--------------------------------------------------------------------------------------
class CNeuralNetworkRemoteControl: public Enki::RemoteControl
{
protected:
	CNeuralNetworkRemoteControl(const CNeuralNetworkRemoteControl &);
	const CNeuralNetworkRemoteControl & operator=(const CNeuralNetworkRemoteControl &);
public:
	CNeuralNetworkRemoteControl():
	  bUp(false),
	  bDown(false),
	  bLeft(false),
	  bRight(false)
	  { }
	virtual ~CNeuralNetworkRemoteControl() { }

	virtual bool isUp();
	virtual bool isDown();
	virtual bool isLeft();
	virtual bool isRight();

	virtual void Clear();
	virtual void Update();

	virtual bool IsConsistent();

	RFW::CFeedForward & GetNeuralNetwork() { return m_FeedForward; }
protected:
	RFW::CFeedForward m_FeedForward;
	bool bUp;
	bool bDown;
	bool bLeft;
	bool bRight;
};

#endif // REMOTECONTROL_H