#ifndef ROBOTNEURALSIMULATORSCENE_H
#define ROBOTNEURALSIMULATORSCENE_H

//--------------------------------------------------------------------------------------
// File: RobotNeuralSimulatorScene.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// Robot Neural Simulator scene header
// CRobotNeuralSimulatorScene class encapsulates all the basic functionality
// of the simulator
//--------------------------------------------------------------------------------------

#include <string>

#include "RFWReportError.h"

namespace RFW
{
	class IEnkiWorld;
	class CFeedForward;
	class CEnkiNeuralRobot;
}

class CVertexBuffer;
class CKeyboard;

class CLog;
class CKeyboardRemoteControl;
class CNeuralNetworkRemoteControl;

class CRobotNeuralSimulatorScene;

//--------------------------------------------------------------------------------------
// CRobotNeuralSimulatorScene class
//--------------------------------------------------------------------------------------
class CRobotNeuralSimulatorScene: public RFW::IReportErrorAble
{
protected:
	CRobotNeuralSimulatorScene(const CRobotNeuralSimulatorScene &);
	const CRobotNeuralSimulatorScene & operator=(const CRobotNeuralSimulatorScene &);
public:
	static const double defaultWidth;
	static const double defaultHeight;
public:
	CRobotNeuralSimulatorScene(CVertexBuffer &, CKeyboard &);
	virtual ~CRobotNeuralSimulatorScene();

	virtual bool Good() { return m_ReportError.Good(); }
	virtual void Error(std::string err) { m_ReportError.Error(err); }
	virtual void ClearError();
	virtual std::string ReportError() { return m_ReportError.ReportError(); }

	virtual void ClearScene();
	virtual void ClearRobot();
	virtual void ClearNetwork();

	virtual bool LoadSceneFromFile(std::string fileName);
	virtual bool LoadRobotFromFile(std::string fileName);
	virtual bool LoadNetworkFromFile(std::string fileName);

	virtual bool StartLog(std::string fileName);
	virtual bool isLogging();
	virtual void Log(float fElapsedTime);
	virtual bool StopLog();

	virtual bool SetRobotToKeyboard();
	virtual bool SetRobotToNeuralNetwork();

	virtual bool IsRobotSetToKeyboard() { return !m_RobotToNeuralNetwork; }
	virtual bool IsRobotSetToNeuralNetwork() { return m_RobotToNeuralNetwork; }

	virtual RFW::IEnkiWorld * GetWorld() { return m_EnkiWorld; }
	virtual RFW::CEnkiNeuralRobot * GetRobot() { return m_EnkiNeuralRobot; }

	virtual void Render();

	virtual void Step(double);
protected:
	CVertexBuffer & m_VertexBuffer;
	CKeyboard & m_Keyboard;
	RFW::CReportError m_ReportError;
	bool m_RobotToNeuralNetwork;
	CLog * m_Log;
	CKeyboardRemoteControl * m_KeyboardRemoteControl;
	CNeuralNetworkRemoteControl * m_NeuralNetworkRemoteControl;
	RFW::IEnkiWorld * m_EnkiWorld;
	RFW::CEnkiNeuralRobot * m_EnkiNeuralRobot;
};

#endif // ROBOTNEURALSIMULATORSCENE_H