#ifndef STATUSBAR_H
#define STATUSBAR_H

//--------------------------------------------------------------------------------------
// File: StatusBar.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// Simple status bar header
// CStatusBar class
//
//--------------------------------------------------------------------------------------

#include <string>

//--------------------------------------------------------------------------------------
// CStatusBar class
//--------------------------------------------------------------------------------------
class CStatusBar
{
public:
	enum Priority {Zero, Low, Normal, High};
public:
	CStatusBar():
	  m_DefaultText(L"Ready"),
	  m_ActualText (L"Ready"),
	  m_ActualPriority(Zero),
	  m_Deadline(0.0f),
	  m_ActualTime(0.0f)
	{ }
	virtual ~CStatusBar() { }

	void SetActualTime(float time) { m_ActualTime = time; }
	void SetText(std::string   text, float time, Priority priority);
	void SetText(std::wstring wtext, float time, Priority priority);
	void SetLowPriorityText(std::string text)    { SetText(text, 2.0f, Low); }
	void SetNormalPriorityText(std::string text) { SetText(text, 2.0f, Normal); }
	void SetHighPriorityText(std::string text)   { SetText(text, 5.0f, High); }
	std::wstring GetText();

	void SetDefaultText(std::wstring wtext) { m_DefaultText = wtext; }
protected:
	std::wstring m_DefaultText;
	std::wstring m_ActualText;
	Priority m_ActualPriority;
	float m_Deadline;

	float m_ActualTime;
};

void CStatusBar::SetText(std::string text, float time, Priority priority)
{
	std::wstring wtext(text.begin(), text.end());
	SetText(wtext, time, priority);
}

void CStatusBar::SetText(std::wstring wtext, float time, Priority priority)
{
	if (priority >= m_ActualPriority)
	{
		m_ActualPriority = priority;
		m_ActualText = wtext;
		m_Deadline = m_ActualTime + time;
	}
}

std::wstring CStatusBar::GetText()
{
	if (m_ActualTime > m_Deadline)
	{
		m_ActualPriority = Zero;
		m_ActualText = m_DefaultText;
	}
	return m_ActualText;
}

#endif // STATUSBAR_H