#ifndef VERTEXBUFFER_H
#define VERTEXBUFFER_H

//--------------------------------------------------------------------------------------
// File: VertexBuffer.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// Vertex buffer for DirectX 9.0 header
// Contains basic classes for managing vertex buffer
//
//--------------------------------------------------------------------------------------

#include "D3DDeviceObserverPattern.h"

class CVertexBuffer;

//--------------------------------------------------------------------------------------
// CVertexBuffer class implements ID3DDeviceObserver interface
//--------------------------------------------------------------------------------------
class CVertexBuffer: public ID3DDeviceObserver
{
public:
	CVertexBuffer();
	virtual ~CVertexBuffer();

	virtual HRESULT OnCreatedDevice( IDirect3DDevice9* pd3dDevice );
	virtual HRESULT OnResetDevice();
	virtual HRESULT OnLostDevice();
	virtual HRESULT OnDestroyDevice();

	HRESULT Clear();
	HRESULT CopyVertices(void *pData, UINT Size);
	HRESULT AddVertices(void *pData, UINT Size);
	HRESULT SetVertexBuffer();

	HRESULT DrawPrimitive(D3DPRIMITIVETYPE PrimitiveType);
	HRESULT DrawPrimitive(D3DPRIMITIVETYPE PrimitiveType, D3DXMATRIX & matTransform);
protected:
	UINT m_Size;
	UINT m_LockedSize;
	IDirect3DVertexBuffer9* m_pVB;
	IDirect3DDevice9* m_pd3dDevice;
};

#endif // VERTEXBUFFER_H