#ifndef VIEWPORT_H
#define VIEWPORT_H

//--------------------------------------------------------------------------------------
// File: Viewport.h
// Author: Peter Cerno <petercerno@gmail.com>
//
// Viewport class header
// CViewport class implements all basic functionality for viewports
//
//--------------------------------------------------------------------------------------

#include "D3DDeviceObserverPattern.h"

class CViewport;

//--------------------------------------------------------------------------------------
// CViewport class
//--------------------------------------------------------------------------------------
class CViewport: public ID3DDeviceObserver
{
public:
	CViewport():
	  m_pd3dDevice(0),
	  m_bUpdateMatrices(false),
	  m_bClippingRect(false),
	  m_bViewportActivated(false),
	  m_fScale(0.0f),
	  m_fX(0.0f),
	  m_fY(0.0f)
	{
		D3DXMatrixIdentity(&m_matrixWorld);
		D3DXMatrixIdentity(&m_matrixView);
		D3DXMatrixIdentity(&m_matrixProjection);
		D3DXMatrixIdentity(&m_oldWorld);
		D3DXMatrixIdentity(&m_oldView);
		D3DXMatrixIdentity(&m_oldProjection);
		m_viewArea.X = 0;
		m_viewArea.Y = 0;
		m_viewArea.Width = 0;
		m_viewArea.Height = 0;
		m_viewArea.MinZ = 0.0f;
		m_viewArea.MaxZ = 1.0f;
		m_oldArea.X = 0;
		m_oldArea.Y = 0;
		m_oldArea.Width = 0;
		m_oldArea.Height = 0;
		m_oldArea.MinZ = 0.0f;
		m_oldArea.MaxZ = 1.0f;
	}
	virtual ~CViewport() { }

	HRESULT OnCreatedDevice( IDirect3DDevice9* pd3dDevice );
	HRESULT OnResetDevice();
	HRESULT OnLostDevice();
	HRESULT OnDestroyDevice();

	HRESULT SetViewport();
	HRESULT ClearViewport(D3DCOLOR Color);
	HRESULT UndoViewport();

	void SetClippingRect(int X, int Y, int Width, int Height);

	void SetToTarget(float Width, float Height);
	void MoveTo(float fX, float fY, float fElapsedTime);
	void MoveBy(float fX, float fY, float fElapsedTime)
	{
		MoveTo(m_fX + fX, m_fY + fY, fElapsedTime);
	}

	void ScaleTo(float fScale, float fElapsedTime);
	void ScaleBy(float fScale, float fElapsedTime)
	{
		ScaleTo(fScale * m_fScale, fElapsedTime);
	}

	bool IsInside(int globalX, int globalY) const;

	void  SetScaling(float fScale) { m_fScale = fScale; m_bUpdateMatrices = true; }
	float GetScaling() const { return m_fScale; }

	void  SetTranslation(float fX, float fY) { m_fX = fX; m_fY = fY; m_bUpdateMatrices = true; }
	float GetTranslationX() const { return m_fX; }
	float GetTranslationY() const { return m_fY; }

	void GetGlobalPoint(int & globalX, int & globalY, float localX, float localY) const;
	void GetGlobalPoint(int & globalX, int & globalY, const D3DXVECTOR2 & localVector) const;
	void GetLocalPoint (int   globalX, int   globalY, float & localX, float & localY) const;
	void GetLocalPoint (int   globalX, int   globalY, D3DXVECTOR2 & localVector) const;
protected:
	IDirect3DDevice9* m_pd3dDevice;
	D3DVIEWPORT9 m_viewArea;
	mutable D3DXMATRIX   m_matrixWorld;
	mutable D3DXMATRIX   m_matrixView;
	mutable D3DXMATRIX   m_matrixProjection;
	mutable D3DXMATRIX   m_matrixProduct;
	mutable D3DXMATRIX   m_matrixProductInverse;
	D3DVIEWPORT9 m_oldArea;
	D3DXMATRIX   m_oldWorld;
	D3DXMATRIX   m_oldView;
	D3DXMATRIX   m_oldProjection;
	mutable bool m_bUpdateMatrices;
	bool         m_bClippingRect;
	bool         m_bViewportActivated;
	float        m_fScale;
	float        m_fX, m_fY;

	void SetMatrices() const;
};

#endif // VIEWPORT_H