/*
    Enki - a fast 2D robot simulator
    Copyright (C) 1999-2005 Stephane Magnenat <nct@ysagoon.com>
    Copyright (C) 2005 Laboratory of Intelligent Systems, EPFL, Lausanne
    See AUTHORS for details

    This program is free software; the authors of any publication 
    arising from research using this software are asked to add the 
    following reference:
    Enki - a fast 2D robot simulator part of the Teem framework
    http://teem.epfl.ch
    Stephane Magnenat <stephane.magnenat@epfl.ch>,
    Markus Waibel <markus.waibel@epfl.ch>
    Laboratory of Intelligent Systems, EPFL, Lausanne.

    You can redistribute this program and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __INTERACTION_H
#define __INTERACTION_H

#include <assert.h>

namespace Enki
{
	class PhysicalObject;
	class Robot;
	class World;

	//! Interacts with another object or wall only up to a certain distance
	class LocalInteraction
	{
	protected:
		LocalInteraction(const LocalInteraction &);
		const LocalInteraction & operator=(const LocalInteraction &);
	protected:
		//! Radius of the local interaction
		double r;

		//! Robots can access protected members me
		friend class Robot;
		//! The physical object that owns the interaction.
		Robot *owner;

	public :
		LocalInteraction(): r(0), owner(0) { }
		//! Destructor
		virtual ~LocalInteraction() { }
		//! Set owner
		virtual void setOwner(Robot * o) { assert(o); owner = o; }
		//! Init at each step
		virtual void init() { }
		//! Interact with object
		virtual void objectStep(double, PhysicalObject *, World *) { }
		//! Interact with walls
		virtual void wallsStep(World *) { }
		//! Finalize at each step
		virtual void finalize(double) { }
		//! Return the range of the interaction
		double getRange() const { return r; }
	};

	//! Interacts with the whole world
	class GlobalInteraction
	{
	protected:
		GlobalInteraction(const GlobalInteraction &);
		const GlobalInteraction & operator=(const GlobalInteraction &);
	protected:
		//! The physical object that owns the interaction.
		Robot *owner;

	public :
		GlobalInteraction(): owner(0) { }
		//! Destructor
		virtual ~GlobalInteraction() { }
		//! Set owner
		virtual void setOwner(Robot * o) { owner = o; }
		//! Init at each step
		virtual void init() { }
		//! Interact with world
		virtual void step(double, World *) { }
		//! Finalize at each step
		virtual void finalize() { }
	};
}
#endif

