/*
    Enki - a fast 2D robot simulator
    Copyright (C) 1999-2005 Stephane Magnenat <nct@ysagoon.com>
    Copyright (C) 2004-2005 Markus Waibel <markus.waibel@epfl.ch>
    Copyright (C) 2005 Laboratory of Intelligent Systems, EPFL, Lausanne
    See AUTHORS for details

    This program is free software; the authors of any publication 
    arising from research using this software are asked to add the 
    following reference:
    Enki - a fast 2D robot simulator part of the Teem framework
    http://teem.epfl.ch
    Stephane Magnenat <stephane.magnenat@epfl.ch>,
    Markus Waibel <markus.waibel@epfl.ch>
    Laboratory of Intelligent Systems, EPFL, Lausanne.

    You can redistribute this program and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma warning(disable : 4530)
#pragma warning(disable : 4996)

#include "AliceCam.h"
#include <iostream>

// Implementation of the Alice 2 pixels camera interaction

namespace Enki
{
	using namespace An;
	
	AliceCam::AliceCam ()
	{
		this->owner = 0;
		this->cvaluestarboard = true;
		this->cvalueport = true;
	}

	void AliceCam::step(double, World *w)
	{
		Vector v1 = Vector(w->w,w->h)-this->owner->pos;
		Vector v2 = Vector(0,w->h)-this->owner->pos;

		// camera port
		Vector dir(cos(this->owner->angle - M_PI/10), sin(this->owner->angle - M_PI/10));

		if ( (dir.cross(v1) < 0) && (dir.cross(v2) > 0))
		{
			// nest
			cvalueport = 1;
		}
		else
		{
			// not nest
			cvalueport = 0;
		}

		// camera starboard
		dir=Vector(cos(this->owner->angle + M_PI/10), sin(this->owner->angle + M_PI/10));

		if ( (dir.cross(v1) < 0) && (dir.cross(v2) > 0))
		{
			// nest
			cvaluestarboard = 1;
		}
		else
		{
			// not nest
			cvaluestarboard = 0;
		}

	}

	double AliceCam::getCValueStarboard()
	{
		return cvaluestarboard;
	}
	double AliceCam::getCValuePort()
	{
		return cvalueport;
	}


}
