/*
    Enki - a fast 2D robot simulator
    Copyright (C) 1999-2005 Stephane Magnenat <nct@ysagoon.com>
    Copyright (C) 2005 Laboratory of Intelligent Systems, EPFL, Lausanne
    See AUTHORS for details

    This program is free software; the authors of any publication 
    arising from research using this software are asked to add the 
    following reference:
    Enki - a fast 2D robot simulator part of the Teem framework
    http://teem.epfl.ch
    Stephane Magnenat <stephane.magnenat@epfl.ch>,
    Markus Waibel <markus.waibel@epfl.ch>
    Laboratory of Intelligent Systems, EPFL, Lausanne.

    You can redistribute this program and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma warning(disable : 4530)
#pragma warning(disable : 4996)

#include "Sbot.h"

// Implementation of the Sbot robot

namespace Enki
{
	using namespace An;
	
	Sbot::Sbot() :
		camera(64)
	{
		camera.setOwner(this);
		sound.setOwner(this);
		mass = 500;
		r = 6;
		addLocalInteraction(&camera);
		addGlobalInteraction(&sound);
		leftSpeed = 0;
		rightSpeed = 0;
	}

	void Sbot::step(double dt)
	{
		// handle physic
		PhysicalObject::step(dt);
		
		// +/- 2% motor noise
		double realRightSpeed = rightSpeed * (0.98 + random.getRange(0.04));
		double realLeftSpeed = leftSpeed * (0.98 + random.getRange(0.04));

		double forwardSpeed = (realRightSpeed + realLeftSpeed) / 2;
		double wheelDist = 5;

		// forward component
		speed = Vector(forwardSpeed * cos(angle), forwardSpeed * sin(angle));

		// angle
		angSpeed = (realRightSpeed-realLeftSpeed)/(2*wheelDist);
	}
	
	unsigned SbotGlobalSound::worldFrequenciesState = 0;
	
	unsigned SbotGlobalSound::getWorldFrequenciesState(void)
	{
		return worldFrequenciesState;
	}
	
	void FeedableSbot::step(double dt)
	{
		Sbot::step(dt);
		
		// clear dEnegry for next step
		energy += dEnergy * dt;
		lastDEnergy = dEnergy;
		dEnergy = 0;
	}
}
