/*
    Enki - a fast 2D robot simulator
    Copyright (C) 1999-2005 Stephane Magnenat <nct@ysagoon.com>
    Copyright (C) 2005 Laboratory of Intelligent Systems, EPFL, Lausanne
    See AUTHORS for details

    This program is free software; the authors of any publication 
    arising from research using this software are asked to add the 
    following reference:
    Enki - a fast 2D robot simulator part of the Teem framework
    http://teem.epfl.ch
    Stephane Magnenat <stephane.magnenat@epfl.ch>,
    Markus Waibel <markus.waibel@epfl.ch>
    Laboratory of Intelligent Systems, EPFL, Lausanne.

    You can redistribute this program and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma warning(disable : 4530)
#pragma warning(disable : 4996)

#include "SbotCam.h"
#include "Sbot.h"
#include <iostream>
#include <algorithm>
#include <math.h>
#include <limits>

// Implementation of the Sbot 1D circular camera

namespace Enki
{
	using namespace An;
	
	SbotCam::SbotCam(unsigned halfPixelCount) :
		zbuffer(halfPixelCount * 2),
		image(halfPixelCount * 2),
		// FIXME
		// This is the clean and logic way to do it
		/*cam0(owner, Point(0, 0), -M_PI/2, M_PI/2, halfPixelCount),
		  cam1(owner, Point(0, 0), M_PI/2, M_PI/2, halfPixelCount)*/
		// This is the stupid backward compatible way, TO BE REMOVED
		cam0(Point(0, 0), 0,  M_PI/2, M_PI/2, halfPixelCount),
		cam1(Point(0, 0), 0, -M_PI/2, M_PI/2, halfPixelCount)
	{
		this->r = std::numeric_limits<double>::max();
	}

	void SbotCam::setOwner(Robot *o)
	{
		cam0.setOwner(o);
		cam1.setOwner(o);
		this->owner = o;
	}

	void SbotCam::objectStep(double dt, PhysicalObject *po, World *w) 
	{
		cam0.objectStep(dt, po, w);
		cam1.objectStep(dt, po, w);
	};

	void SbotCam::init()
	{
		cam0.init();
		cam1.init();
	}
	
	void SbotCam::wallsStep(World *w)
	{
		cam0.wallsStep(w);
		cam1.wallsStep(w);
	}
	
	void SbotCam::finalize(double dt)
	{
		cam0.finalize(dt);
		cam1.finalize(dt);
		size_t camPixelCount = cam0.zbuffer.size();
		std::copy(&cam0.zbuffer[0], &cam0.zbuffer[camPixelCount], &zbuffer[0]);
		std::copy(&cam1.zbuffer[0], &cam1.zbuffer[camPixelCount], &zbuffer[camPixelCount]);
		std::copy(&cam0.image[0], &cam0.image[camPixelCount], &image[0]);
		std::copy(&cam1.image[0], &cam1.image[camPixelCount], &image[camPixelCount]);
	}
	
	void SbotCam::setFogConditions(bool useFog, double density, An::Color threshold)
	{
		cam0.useFog = useFog;
		cam0.fogDensity = density;
		cam0.lightThreshold = threshold;
		cam1.useFog = useFog;
		cam1.fogDensity = density;
		cam1.lightThreshold = threshold;
	}
	
	void SbotCam::setRange(double range)
	{
		this->r = range;
		owner->sortLocalInteractions();
	}
	
	void SbotCam::setPixelOperationFunctor(PixelOperationFunctor *pixelOperationFunctor)
	{
		cam0.pixelOperation = pixelOperationFunctor;
		cam1.pixelOperation = pixelOperationFunctor;
	}
}


